@@ -142,6 +142,11 @@ HEADER_KEYS_V1_4 = [
 # Tor version will be obtained from the state file, so it won't be pass as an
 # argument, but will be self-initialized.
 HEADER_KEYS_V1_4_TO_INIT = ["tor_version"]
+# Network averages: network bandwidth mean and network bandwidth filtered.
+HEADER_KEYS_V1_7 = [
+    "mu",
+    "muf",
+]
 
 # KeyValues that are initialized from arguments, not self-initialized.
 HEADER_INIT_KEYS = (
@@ -161,6 +166,7 @@ HEADER_UNORDERED_KEYS = (
     + HEADER_KEYS_V1_2
     + HEADER_KEYS_V1_4
     + HEADER_KEYS_V1_4_TO_INIT
+    + HEADER_KEYS_V1_7
 )
 # List of all the KeyValues currently being used to generate the file
 HEADER_ALL_KEYS = HEADER_KEYS_V1_1_ORDERED + HEADER_UNORDERED_KEYS
@@ -254,8 +260,22 @@ BWLINE_KEYS_V1_4 = [
     # Added in #29853.
     "under_min_report",
 ]
+BWLINE_KEYS_V1_6 = [
+    "xoff_recv",
+    "xoff_sent",
+]
+# Relays' ratios: stream and stream filtered
+BWLINE_KEYS_V1_7 = [
+    "r_strm",
+    "r_strm_filt",
+]
 BWLINE_KEYS_V1 = (
-    BWLINE_KEYS_V0 + BWLINE_KEYS_V1_1 + BWLINE_KEYS_V1_2 + BWLINE_KEYS_V1_4
+    BWLINE_KEYS_V0
+    + BWLINE_KEYS_V1_1
+    + BWLINE_KEYS_V1_2
+    + BWLINE_KEYS_V1_4
+    + BWLINE_KEYS_V1_6
+    + BWLINE_KEYS_V1_7
 )
 # NOTE: tech-debt: assign boolean type to vote and unmeasured,
 # when the attributes are defined with a type, as stem does.
@@ -270,6 +290,7 @@ BWLINE_INT_KEYS = (
     ]
     + BWLINE_KEYS_V1_2
     + BWLINE_KEYS_V1_4
+    + BWLINE_KEYS_V1_6
 )
 # This is boolean, not int.
 BWLINE_INT_KEYS.remove("consensus_bandwidth_is_unmeasured")
@@ -284,7 +305,6 @@ def round_sig_dig(n, digits=PROP276_ROUND_DIG):
     n must be less than or equal to 2**73, to avoid floating point errors.
     """
     digits = int(digits)
-    assert digits >= 1
     if n <= 1:
         return 1
     digits_in_n = int(math.log10(n)) + 1
@@ -335,9 +355,6 @@ class V3BWHeader(object):
     """
 
     def __init__(self, timestamp, **kwargs):
-        assert isinstance(timestamp, str)
-        for v in kwargs.values():
-            assert isinstance(v, str)
         self.timestamp = timestamp
         # KeyValues with default value when not given by kwargs
         self.version = kwargs.get("version", SPEC_VERSION)
@@ -437,7 +454,6 @@ class V3BWHeader(object):
         :param list lines: list of lines to parse
         :returns: tuple of V3BWHeader object and non-header lines
         """
-        assert isinstance(lines, list)
         try:
             index_terminator = lines.index(TERMINATOR)
         except ValueError:
@@ -462,7 +478,6 @@ class V3BWHeader(object):
         :param str text: text to parse
         :returns: tuple of V3BWHeader object and non-header lines
         """
-        assert isinstance(text, str)
         return self.from_lines_v1(text.split(LINE_SEP))
 
     @classmethod
@@ -471,7 +486,6 @@ class V3BWHeader(object):
         :param list lines: list of lines to parse
         :returns: tuple of V3BWHeader object and non-header lines
         """
-        assert isinstance(lines, list)
         h = cls(lines[0])
         # last line is new line
         return h, lines[1:-1]
@@ -662,6 +676,10 @@ class V3BWHeader(object):
         for k, v in exclusion_dict.items():
             setattr(self, k, str(v))
 
+    def add_net_bw_avgs(self, mu, muf):
+        self.mu = "{:d}".format(round(mu))
+        self.muf = "{:d}".format(round(muf))
+
 
 class V3BWLine(object):
     """
@@ -677,8 +695,6 @@ class V3BWLine(object):
     """
 
     def __init__(self, node_id, bw, **kwargs):
-        assert isinstance(node_id, str)
-        assert node_id.startswith("$")
         self.node_id = node_id
         self.bw = bw
         # For now, we do not want to add ``bw_filt`` to the bandwidth file,
@@ -750,6 +766,18 @@ class V3BWLine(object):
                 ts.update(r.relay_recent_measurement_attempt)
         kwargs["relay_recent_measurement_attempt_count"] = str(len(ts))
 
+        ts = set([])
+        for r in results:
+            if getattr(r, "xoff_recv", None):
+                ts.update(r.xoff_recv)
+        kwargs["xoff_recv"] = str(len(ts))
+
+        ts = set([])
+        for r in results:
+            if getattr(r, "xoff_sent", None):
+                ts.update(r.xoff_sent)
+        kwargs["xoff_sent"] = str(len(ts))
+
         success_results = [r for r in results if isinstance(r, ResultSuccess)]
 
         # NOTE: The following 4 conditions exclude relays from the bandwidth
@@ -879,12 +907,10 @@ class V3BWLine(object):
 
     @classmethod
     def from_data(cls, data, fingerprint):
-        assert fingerprint in data
         return cls.from_results(data[fingerprint])
 
     @classmethod
     def from_bw_line_v1(cls, line):
-        assert isinstance(line, str)
         kwargs = dict(
             [
                 kv.split(KEYVALUE_SEP_V1)
@@ -1050,7 +1076,9 @@ class V3BWLine(object):
         spec v1.X.X.
         """
         bw_keyvalue_str = [
-            KEYVALUE_SEP_V1.join([k, str(v)])
+            KEYVALUE_SEP_V1.join(
+                [k, str(v) if not isinstance(v, float) else "{:.2f}".format(v)]
+            )
             for k, v in self.bw_keyvalue_tuple_ls
         ]
         return bw_keyvalue_str
@@ -1181,13 +1209,14 @@ class V3BWFile(object):
             cls.warn_if_not_accurate_enough(bw_lines, scale_constant)
             # log.debug(bw_lines[-1])
         elif scaling_method == TORFLOW_SCALING:
-            bw_lines = cls.bw_torflow_scale(
+            bw_lines, mu, muf = cls.bw_torflow_scale(
                 bw_lines_raw,
                 torflow_obs,
                 torflow_cap,
                 round_digs,
                 router_statuses_d=router_statuses_d,
             )
+            header.add_net_bw_avgs(mu, muf)
             # log.debug(bw_lines[-1])
             # Update the header and log the progress.
             min_perc = cls.update_progress(
@@ -1412,9 +1441,9 @@ class V3BWFile(object):
                 continue
 
             # Torflow's scaling
-            ratio_stream = l.bw_mean / mu
-            ratio_stream_filtered = l.bw_filt / muf
-            ratio = max(ratio_stream, ratio_stream_filtered)
+            l.r_strm = l.bw_mean / mu
+            l.r_strm_filt = l.bw_filt / muf
+            ratio = max(l.r_strm, l.r_strm_filt)
 
             # Assign it to an attribute, so it's not lost before capping and
             # rounding
@@ -1443,7 +1472,11 @@ class V3BWFile(object):
             bw_scaled = min(hlimit, l.bw)
             # round and convert to KB
             l.bw = kb_round_x_sig_dig(bw_scaled, digits=num_round_dig)
-        return sorted(bw_lines_tf, key=lambda x: x.bw, reverse=reverse)
+        return (
+            sorted(bw_lines_tf, key=lambda x: x.bw, reverse=reverse),
+            mu,
+            muf,
+        )
 
     @staticmethod
     def read_number_consensus_relays(consensus_path):
@@ -1496,8 +1529,6 @@ class V3BWFile(object):
         # network status or descriptors?
         # It will not be updated to the last consensus, but the list of
         # measured relays is not either.
-        assert isinstance(number_consensus_relays, int)
-        assert isinstance(num_bw_lines, int)
         statsd = {}
         statsd["number_eligible_relays"] = num_bw_lines
         statsd["number_consensus_relays"] = number_consensus_relays
